<?php

namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Util\Messages;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Repository\DataExportation as DataExportationRepository;
use Teknisa\Model\Integracao\Repositories as IntegracaoRepositories;
use Teknisa\Model\Geral\Repositories as GeralRepositories;
use Teknisa\Libs\Util\Environment;

class DataExportation {
    const NRORG_COLUMN = 'NRORG';
    const NEW_CODE_TABLE_NAME = 'NOVOCODIGO';
    const DOUBLE_BREAK_LINE = '

';

    const IS_ORGANIZATION_SAAS      = 'S';
    const IS_ORGANIZATION_NOT_SAAS  = 'N';

    /** @var EntityManager **/
    protected $entityManager;

    /** @var DataExportationRepository $dataExportationRepository **/
    protected $dataExportationRepository;

    /** @var IntegracaoRepositories\Tablayoutexp $repositoryTablayoutexp **/
    protected $repositoryTablayoutexp;

    /** @var GeralRepositories\Tabela $repositoryTabela **/
    protected $repositoryTabela;

    /** @var Environment $environment */
    private $environment;

    /** @var Utilities */
    private $utilities;

    protected $exclusiveOrgColumns      = array( 'NRORGULTATU', 'NRORGINCLUSAO' );
    protected $exclusiveDateColumns     = array( 'DTINCLUSAO', 'DTULTATU' );
    protected $exclusiveOperatorColumns = array( 'CDOPERINCLUSAO', 'CDOPERULTATU' );

    /**
     * @param EntityManager $entityManager
     * @param DataExportationRepository $dataExportationRepository
     * @param Environment $environment
     * @param Utilities $utilities
     */
    public function __construct(EntityManager $entityManager, DataExportationRepository $dataExportationRepository,
                                Environment $environment, Utilities $utilities) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->dataExportationRepository = $dataExportationRepository;

        $this->repositoryTablayoutexp = $this->entityManager->getRepository(Utilities::INTEGRACAO_ENTITY_DIR . 'Tablayoutexp');
        $this->repositoryTabela = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Tabela');
        $this->utilities = $utilities;
    }

    /**
     * @param $layoutId
     * @param $originOrganization
     * @param $destinyOrganization
     * @param $destinyIdOrgSaas
     * @throws \Exception
     * @return array
     */
    public function getDataExportationInfo($layoutId, $originOrganization, $destinyOrganization, $destinyIdOrgSaas) {
        if($originOrganization == $destinyOrganization) {
            throw new \Exception(Messages::ERR_ORIGIN_ORG_EQUAL_DESTINY_ORG);
        }

        $tableLayouts = $this->repositoryTablayoutexp->getTableLayoutExportationByLayoutCodeAndOrgId($layoutId,
            $this->environment->getCurrentOrganizationId());
        $tables = array_map(function ($tableLayout) {
            return $tableLayout['NMTABELABD'];
        }, $tableLayouts);

        $destinyOrgDbUser = Utilities::getOrgDbUserName($destinyOrganization);
        if($destinyIdOrgSaas == self::IS_ORGANIZATION_NOT_SAAS) {
            $entityManager = InstanceProvider::getDBConnection()->getOrgEntityManager(null, true, true);
            $this->dataExportationRepository->setEntityManager($entityManager);
        }
        $this->validateNotEmptyTables($tables, $destinyOrgDbUser, $destinyOrganization);

        return $tables;
    }

    /**
     * @param $tableName
     * @param $originOrg
     * @param $destinyOrg
     * @param $destinyIdOrgSaas
     * @throws \Exception
     * @return array
     */
    public function exportTable($tableName, $originOrg, $destinyOrg, $destinyIdOrgSaas) {
        $originOrgDbUser  = Utilities::getOrgDbUserName($originOrg);
        $destinyOrgDbUser = Utilities::getOrgDbUserName($destinyOrg);

        if($destinyIdOrgSaas == self::IS_ORGANIZATION_NOT_SAAS) {
            $entityManager = InstanceProvider::getDBConnection()->getOrgEntityManager(null, true, true);
            $this->dataExportationRepository->setEntityManager($entityManager);
        }

        $rowsExported     = $this->doExportTable($tableName, $originOrg, $destinyOrg, $originOrgDbUser, $destinyOrgDbUser);
        $result           = str_replace('{{rowCount}}', $rowsExported, Messages::SUC_ROWS_EXPORTED);
        $resultRevert     = '';
        $revertExecuted   = false;
        try {
            $sequenceUpdated = $this->updateTableSequential($tableName, $originOrg, $destinyOrg, $originOrgDbUser, $destinyOrgDbUser);
        } catch(\Exception $e) {
            $resultRevert = $this->revertTableExportation($tableName, $destinyOrg);
            $resultRevert = $e->getMessage() . self::DOUBLE_BREAK_LINE . $resultRevert[0];
        }

        if(!empty($resultRevert)) {
            $result .= self::DOUBLE_BREAK_LINE . $resultRevert;
            $revertExecuted = true;
        } else if (!empty($sequenceUpdated)) {
            $result .= self::DOUBLE_BREAK_LINE . $sequenceUpdated;
        }

        return array($result, $revertExecuted);
    }

    /**
     * @param $tableName
     * @param $destinyOrg
     * @return array
     */
    public function revertTableExportation($tableName, $destinyOrg) {
        $destinyOrgDbUser = Utilities::getOrgDbUserName($destinyOrg);
        $resultRevert = $this->dataExportationRepository->revertTableExportation($destinyOrg, $tableName, $destinyOrgDbUser);
        return [str_replace('{{rowCount}}', $resultRevert->rowCount(), Messages::SUC_ROWS_REVERTED)];
    }

    /**
     * @param $fileName
     * @param $logContent
     * @return array
     */
    public function createDataExportationLog($fileName, $logContent) {
        file_put_contents($this->utilities->getLogFileDir($fileName), $logContent);
        return array(true);
    }

    /**
     * @param $fileName
     */
    public function downloadDataExportationLog($fileName) {
        Utilities::downloadAndRemoveFile($this->utilities->getLogFileDir($fileName), $fileName);
    }

    /**
     * @param $columns
     * @param $destinyOrg
     * @return string
     */
    private function getColumnValues($columns, $destinyOrg) {
        $columnValues = array();
        foreach($columns as $column) {
            if(in_array($column, $this->exclusiveDateColumns)) {
                $columnValues[] = "SYSDATE AS " . $column;
            } else if(in_array($column, $this->exclusiveOperatorColumns)) {
                $columnValues[] = "'" . $this->environment->getCurrentUserId() . "' AS " . $column;
            } else if(in_array($column, $this->exclusiveOrgColumns)) {
                $columnValues[] = $this->environment->getCurrentOrganizationId() . " AS " . $column;
            } else if($column == self::NRORG_COLUMN) {
                $columnValues[] = $destinyOrg . " AS " . $column;
            } else if(substr($column, 0, 5) == self::NRORG_COLUMN) {
                $columnValues[] = "NULL AS " . $column;
            } else {
                $columnValues[] = $column;
            }
        };

        return join(', ', $columnValues);
    }

    /**
     * @param $tableColumns
     * @return array
     */
    private function mappingTableColumns($tableColumns) {
        return array_map(function ($tableColumn) {
            return $tableColumn['COLUMN_NAME'];
        }, $tableColumns);
    }

    /**
     * @param $tableNames
     * @param $dbUser
     * @param $orgId
     * @throws \Exception
     */
    private function validateNotEmptyTables($tableNames, $dbUser, $orgId) {
        $notEmptyTables = [];
        foreach($tableNames as $tableName) {
            $tableRegisters = $this->dataExportationRepository->getTableRegisters($tableName, $dbUser, $orgId);
            if($tableRegisters['COUNT'] > 0) {
                $notEmptyTables[] = $tableName;
            }
        }

        if(!empty($notEmptyTables)) {
            throw new \Exception(str_replace('{{tables}}', join('"<br/>', $notEmptyTables), Messages::ERR_NOT_EMPTY_TABLES_DATA_EXPORTATION));
        }
    }

    private function updateTableSequential($tableName, $originOrg, $destinyOrg, $originOrgDbUser, $destinyOrgDbUser) {
        $tableInfo = $this->repositoryTabela->getTableByName($tableName, $this->environment->getCurrentOrganizationId());
        $result = null;
        if(!empty($tableInfo) && isset($tableInfo['DSIDSEQUENCIAL']) && !empty($tableInfo['DSIDSEQUENCIAL'])) {
            $sequential = $tableInfo['DSIDSEQUENCIAL'];
            if(strpos($sequential, '[SEQ]') !== false) {
                $sequence = trim(str_replace('[SEQ]', '', $sequential));
                $lastSequenceNumber = $this->dataExportationRepository->getSequenceLastNumber($originOrgDbUser, $sequence);
                $lastSequenceNumber = !empty($lastSequenceNumber) && isset($lastSequenceNumber['LAST_NUMBER']) ?
                    intval($lastSequenceNumber['LAST_NUMBER']) : 1;
                $this->dataExportationRepository->callSequenceNewValueProcedure($destinyOrgDbUser, $sequence, $lastSequenceNumber);
                $result = Messages::SUC_SEQUENCE_UPDATED;
            } else {
                $newCodeExportationFilter = "AND CDCONTADOR = '" . $sequential . "'";
                try {
                    $rowsExported = $this->doExportTable(self::NEW_CODE_TABLE_NAME, $originOrg, $destinyOrg,
                        $originOrgDbUser, $destinyOrgDbUser, $newCodeExportationFilter);
                } catch(\Exception $e) {
                    if(strpos($e->getMessage(), 'PK_NOVOCODIGO')) {
                        $resultUpdate = $this->dataExportationRepository->updateNewCode(self::NEW_CODE_TABLE_NAME, $originOrg,
                            $destinyOrg, $sequential, $originOrgDbUser, $destinyOrgDbUser);
                        $rowsExported = $resultUpdate->rowCount();
                    } else {
                        throw $e;
                    }
                }
                $result = str_replace('{{rowCount}}', $rowsExported, Messages::SUC_ROWS_NEW_CODE_EXPORTED);
            }
        }

        return $result;
    }

    /**
     * @param $tableName
     * @param $originOrg
     * @param $destinyOrg
     * @param $originOrgDbUser
     * @param $destinyOrgDbUser
     * @param $extraExportationFilter
     * @return int
     */
    private function doExportTable($tableName, $originOrg, $destinyOrg, $originOrgDbUser, $destinyOrgDbUser, $extraExportationFilter = '') {
        $tableColumns = $this->dataExportationRepository->getTableColumns($tableName, $originOrgDbUser);
        $tableColumns = $this->mappingTableColumns($tableColumns);
        $columnValues = $this->getColumnValues($tableColumns, $destinyOrg);
        $resultExport = $this->dataExportationRepository->exportTable($originOrg, $tableName,
            $originOrgDbUser, $destinyOrgDbUser, join(', ', $tableColumns), $columnValues, $extraExportationFilter);
        return $resultExport->rowCount();
    }
}